/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: msgq_ut.c,v $
 * Revision 1.1.9.2  1996/02/17  23:29:16  marty
 * 	Update OSF copyright year
 * 	[1996/02/17  22:46:02  marty]
 *
 * Revision 1.1.9.1  1995/12/11  22:51:02  root
 * 	Submit OSF/DCE 1.2.1
 * 	[1995/12/11  22:23:40  root]
 * 
 * Revision 1.1.4.2  1993/02/05  15:28:27  cjd
 * 	Embedded copyright notice
 * 	[1993/02/05  14:49:22  cjd]
 * 
 * Revision 1.1.2.3  1992/03/18  21:03:49  eperkins
 * 	Corrected includes
 * 	[1992/03/18  20:52:50  eperkins]
 * 
 * Revision 1.1.2.2  1992/02/13  21:40:04  eperkins
 * 	Removed returnValue because of multiple define issues
 * 	[1992/02/13  21:39:49  eperkins]
 * 
 * Revision 1.1  1992/01/19  14:34:23  devrcs
 * 	Initial revision
 * 
 * $EndLog$
 */
/*
 *
 *
 * FILE_NAME: msgq_ut.c
 *
 * COMPONENT_NAME: dce.systest
 *
 * FUNCTIONS:	MSGprintError()
 *
 *
 * ORIGINS: 27
 *
 *
 * (C) COPYRIGHT International Business Machines Corp. 1991
 * All Rights Reserved
 *
 *
 * DESCRIPTION:
 *      This file contains utility functions needed by all the other msg q
 *	modules.
 *
 * HISTORY: 
 * 06/24/91	Wayne Mock	Initial design/code
 * 08/30/91	Wayne Mock	Updated the documentation
 */
#include <msgq.h>
int returnValue = 0;              /* global error condition */
/*****************************************************************************
 * MSGprintError()
 *
 * DESCRIPTION
 *	Used to print fatal and non-fatal error messages.  If the message
 *	is fatal, the program exits with a non-zero return code.  It also
 *	uses perror() to print a system call error message if appropriate.
 *
 * INPUT PARAMETERS
 *	enum msgerr_t type	Either a FATAL error or NON_FATAL error.
 *	char* msg		Part of the error message to print.
 *	int err			The error code generated by the bad system
 *				call.
 *
 * OUTPUT PARAMETERS
 *	none.
 *
 * RETURN VALUE
 *	none.
 *
 ****************************************************************************/
void
MSGprintError
#ifdef __NO_PROTOTYPES__
( type, msg, err )
enum msgerr_t type;
char* msg;
int err;
#else
(
enum msgerr_t type,
char* msg,
int err
)
#endif
{	static char errBuf[2048];

/*
 * if it is possible to use perror(), errno actually has a value, use it.
 */
	if( err != 0 && err != -1 )
	{	(void)sprintf( errBuf, "%s(errno=%d)", msg, err );
		errno = err;
		perror( errBuf );
		returnValue = err;
	}
	else
		returnValue = 1;

/*
 * MSG_NON_FATAL_ERRORs just print the given message.
 */
	if( type == MSG_NON_FATAL_ERROR )
	{	fprintf( stderr, "%s\n", msg );
	}
	else
/*
 * The type is MSG_FATAL_ERROR exit the program.
 */
	{	fprintf( stderr, "%s\n", msg );
		exit( returnValue );
	}

} /* end printError() */


